scene = {}
scene.__index = scene


sqrt2 = math.sqrt(2)
sqrt3 = math.sqrt(3)
local canvas = love.graphics.newCanvas(5000,5000)

function scene.new()
	isoDestinationX = 0
	isoDestinationY = 0
	return setmetatable({
		backgroundSizeX = 0,
		backgroundSizeY = 0,
		background = {},
		objectSize = 0,
		objects = {},
		drawOrder = {}}, scene)

end



--Adds an object to a scene
function scene:addToScene(object)

	self.objectSize = self.objectSize + 1
	self.objects[self.objectSize] = object



	self.drawOrder[self.objectSize] = object
	if self.objectSize > 1 then
		for x = self.objectSize, 2, -1 do

			if self.drawOrder[x-1]:getY() > self.drawOrder[x]:getY() then
				local temp = self.drawOrder[x-1]
				self.drawOrder[x-1] = self.drawOrder[x]
				self.drawOrder[x] = temp
			else
				break
			end
		end
	end

end



--Removes the specified object from a scene
function scene:removeFromScene(objectToRemove)
	for x = 1, self.objectSize do
		if self.objects[x] == objectToRemove then
			for y = x, self.size do
				self.objects[y] = self.objects[y+1]
			end
			self.objectSize = self.objectSize -1
			return
		end
	end
end


--Creates the background for the scene
function scene:createBackground(map,x,y)

	self.backgroundSizeX = x or 30
	self.backgroundSizeY = y or x or 30
	dirt = {}
	dirt.image = love.graphics.newImage("Images/soil2.png")

	canvas = love.graphics.newCanvas(5000,5000)
	love.graphics.setCanvas(canvas)
		for x = 0,self.backgroundSizeX-1 do
			for y = 0,self.backgroundSizeY-1 do
				--[[if grid[x][y] == 1 then
					love.graphics.draw(grass.image,
					 (((y/sqrt2)+(x/sqrt2)) * 100 )  ,
					 (((y/sqrt2)-(x/sqrt2)) * 66   + 1500) )
				else --]]-- grid[x][y] == 2
					love.graphics.draw(dirt.image,
					 (((y/sqrt2)+(x/sqrt2)) * 105 )  ,
					 (((y/sqrt2)-(x/sqrt2)) * 60.2   + 1500) )
				--end
			end
		end
	love.graphics.setCanvas()

end




--Updates all objects and their draw order
function scene:update(dt)
	for x=1, self.objectSize do
		local object = self.objects[x]
		self.objects[x]:update(dt)--updates position, velocities, etc.

		--If the object is moving it checks to make sure the object does not go outside the map.
		if not object:getStatic() then

			--Gets the position of the moving object and it's x and y coordinates
			--If the coords are outside the grid then the object has it's velocity and position fixed accordingly.
			local position = object:getPosition()
			local mapx = object:getX()
			local mapy = object:getY()

			if mapx < 0 or mapx > self.backgroundSizeX then
				local velocity = object:getVelocity()
				local speed = object:getSpeed()
				local newVelocity = velocity:projectOn(Vector.new(0,1))
				local newPosition = (-1*velocity*speed) + position
				object:setPosition((-1*velocity*speed) + position)
				object:setVelocity(newVelocity)
				object:setPosition((newVelocity * speed) + object:getPosition() )
				camera:setPosition(self.objects[x]:getX(), self.objects[x]:getY())
			end
			position = object:getPosition()
			mapx = object:getX()
			mapy = object:getY()
			if mapy < 0 or mapy > self.backgroundSizeY  then
				local velocity = object:getVelocity()
				local speed = object:getSpeed()
				local newVelocity = velocity:projectOn(Vector.new(1,0))
				local newPosition = (-1*velocity*speed) + position
				object:setPosition((-1*velocity*speed) + position)
				object:setVelocity(newVelocity)
				object:setPosition((newVelocity * speed) + object:getPosition() )
				camera:setPosition(self.objects[x]:getX(), self.objects[x]:getY())
			end
			self.objects[x] = object
		end	 
	end



	--[[After all objects are updated then the draw order is updated. If any y positions have changed the draw order will change accordingly.
		Known possible problem: If a y coord does change it only moves the object 1 position upward in index, not to the correct index. 	
							    However, since the the update occurs almost 60 times a second then it is possible for the object to
	 						    change index 60 times. So if any object makes a large jump in y the movement should be smooth.
	--]]
	for x = 1, self.objectSize do
		if self.drawOrder[x-1] ~= nil then
			local firstx = self.drawOrder[x-1]:getX()
			local firsty = self.drawOrder[x-1]:getY()
			local firstisoy = ((firsty/sqrt2)-(firstx/sqrt2)) * 60.2   + 1500
			local secondx = self.drawOrder[x]:getX()
			local secondy = self.drawOrder[x]:getY()
			local secondisoy = ((secondy/sqrt2)-(secondx/sqrt2)) * 60.2   + 1500

			if firstisoy > secondisoy then
				local temp = self.drawOrder[x-1]
				self.drawOrder[x-1] = self.drawOrder[x]
				self.drawOrder[x] = temp
			end
		end
	end
end


function scene:drawBackground()
	dirt = {}
	dirt.image = love.graphics.newImage("Images/soil2.png")
		for x = 0,29 do
			for y = 0,29 do
				--[[if grid[x][y] == 1 then
					love.graphics.draw(grass.image,
					 (((y/sqrt2)+(x/sqrt2)) * 100 )  ,
					 (((y/sqrt2)-(x/sqrt2)) * 66   + 1500) )
				else --]]-- grid[x][y] == 2
					love.graphics.draw(dirt.image,
					 (((y/sqrt2)+(x/sqrt2)) * 105 )  ,
					 (((y/sqrt2)-(x/sqrt2)) * 60.2   + 1457) )
				--end

			end
		end


		
end

function scene:drawObjects()
	for x = 1, self.objectSize do
		local xcoord = self.drawOrder[x]:getX()
		local ycoord = self.drawOrder[x]:getY()
		self.drawOrder[x]:draw((((ycoord/sqrt2)+(xcoord/sqrt2)) * 105 ),(((ycoord/sqrt2)-(xcoord/sqrt2)) * 60.2   + 1500))
	end
end


function scene:draw()
	self:drawBackground()
	self:drawObjects()
end



--needs to be rewritten in new format
--[[
function scene:assignSceneColor(object)
	keyColorNum = keyColorNum + 1
	self[self.objectSize].canvasImage = {}
	self[self.objectSize].canvasImage.keyColor = keyColorNum
	self[self.objectSize].canvasImage.imageData = self[self.objectSize].image:getData()
	--will change the image of the new object to the color associated with it's key color. For example, if the
	--key color is 5 then the object's color will be (0,0,5) on an rgb scale.
	--Possible problem: if self.objectSize becomes >255 then I will have to mod and divide by 255 to obtain a g value.
	local function changeColor(x,y,r,g,b,a)
		return 0,0,keyColorNum,a
	end

	self[self.objectSize].canvasImage.imageData:mapPixel(changeColor)
	self[self.objectSize].canvasImage.image = love.graphics.newImage(self[self.objectSize].canvasImage.imageData)

	love.graphics.setCanvas(colorCanvas)
		love.graphics.draw(self[self.objectSize].canvasImage.image, 
			self[self.objectSize].position.x-self[self.objectSize].image:getWidth()/2, 
			self[self.objectSize].position.y-self[self.objectSize].image:getHeight()/2)
	love.graphics.setCanvas()
end

--gets the color at the pixel x,y on the colorCanvas. 
function scene:getAssignedColor(x,y)
	r,g,b,a = colorCanvas:getPixel(camera.position.x + x - love.graphics.getWidth()/2, 
		camera.position.y + y - love.graphics.getHeight()/2)

	for x = 1, self.objectSize do
		if self[x].canvasImage.keyColor == b then 
		end
	end

	return r,g,b,a
end
--]]


setmetatable(scene, { __call = function(_, ...) return scene.new(...) end })